/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "cipiu.h"
#include "gameobj.h"
#include "screen.h"
#include "spritesdef.h"
#include "fldefs.h"

extern SPR_ATTR attr[];
extern int minmaxrndint(int,int);
extern FIXED minmaxrnd(FIXED,FIXED);

// PRIV
int cipiu_guess_winner(CIPIU* c, GAME* g, BALL* bs);
int cipiu_guess_where_to_send(CIPIU* c, GAME* g, BALL* b);
void cipiu_think(CIPIU* c, GAME *g, BALL* bs);
void cipiu_move(CIPIU* c, GAME* g, BALL* bs);

void cipiu_muscle_control(CIPIU* cipiu, CIPIU_MUSCLE_CONTROL c);

void cipiu_init(CIPIU* c, void* vp){
  c->p =(PLAYER*) vp;
  c->timeout = 0;
  c->target = lop_a1;
  c->fire_bonus = 0;
  c->default_pos[X] = c->p->pos[X];
  c->default_pos[Y] = POS_BOTTOM+minmaxrnd(toFIXED(-10.0),toFIXED(0.0));
  c->muscle_timeout[cc_left] = 0;
  c->muscle_timeout[cc_right] = 0;
  c->muscle_timeout[cc_up] = 0;
  c->muscle_timeout[cc_b] = 0;
  c->bonus_timeout = 1000;
}

void cipiu_play(CIPIU* c,void* vg,void* vb){
  GAME* g = (GAME*) vg;
  BALL* b = (BALL*) vb;
  int i;

  // muscle timeout update
  for (i=0;i<cc_last;i++){
    c->muscle_timeout[i] -= 1;
    if ((c->muscle_timeout[i])<0) 
      (c->muscle_timeout[i]) = 0;
  }

  c->bonus_timeout = c->bonus_timeout-1;
  
  // thinking 
  if (c->timeout>0){
    c->timeout--;
  } else {
    cipiu_think(c,g,b); // think now 
    c->timeout = 10; // next turn, cipiu won't really think.
  }

  // action
  cipiu_move(c,g,b);    // move
}

void cipiu_think_now(CIPIU* c){
  c->timeout = 0;
}


/*
 *   PRIVATE
 */

// fast and simple movement interpolation returns <true> if cipiu is expected to win
int cipiu_guess_winner(CIPIU* c, GAME* g, BALL* b){
  int glt;       // guessed losing team
  FIXED gix;     // guessed x of impact point
  int guesses_winner; 

  gix = b->pos[X]+slMulFX(toFIXED(3.0),b->speed[X]);; // *guess*
  glt = net_get_team_from_x(g->net,g->n_net,gix);

  guesses_winner = glt-(c->p->team);

  return (guesses_winner);
}

// return team number to 
int cipiu_guess_where_to_send(CIPIU* c, GAME* g, BALL* b){
  int team;
  if ((g->n_team)==2){
    team = ((c->p->team) == 0)?1:0;
  } else if ((g->n_team)==3){
    int ta,otb,otc;
    ta = (c->p->team);
    otb = (ta == 0)?1:0;
    otc = (ta == 2)?1:2;
    team = ((g->team[otb].points)>(g->team[otc].points))?otb:otc;
  } else { // just in case...
    team = (c->p->team);
  }
  return (team);
}

int is_between(FIXED a,FIXED b, FIXED c){
  int ok;
  ok = ((a<b)&&(b<c));
  return (ok);
}

// brain : decides where to go.
void cipiu_think(CIPIU* c,GAME* g, BALL* bs){

  FIXED dx,dy,adx,ady;
  BALL* b = (BALL*) &(bs[0]);
  PLAYER* p = c->p;
  int cipiu_guesses_he_wins, cipiu_is_at_home, cipiu_has_ball_in_sight;
  int cipiu_guesses_where_to_send, cipiu_is_just_under_the_ball, cipiu_is_just_over_the_ball;
  int cipiu_is_just_at_right_of_ball,cipiu_is_just_at_left_of_ball,cipiu_guesses_ball_is_too_fast;
  FIXED lim;
  dx = p->pos[X] - b->pos[X];  
  dy = p->pos[Y] - b->pos[Y];
 
  adx = (dx>toFIXED(0.0))?dx:slMulFX(toFIXED(-1.0),dx);
  ady = (dy>toFIXED(0.0))?dy:slMulFX(toFIXED(-1.0),dy);

  p->options.go_trough_net = (1==1);

  cipiu_guesses_he_wins = cipiu_guess_winner(c,g,b);
  cipiu_guesses_where_to_send = cipiu_guess_where_to_send(c,g,b);
  cipiu_guesses_ball_is_too_fast = ((is_between(toFIXED(-6.0),b->speed[X],toFIXED(-3.0)))
				   || (is_between(toFIXED(3.0),b->speed[X],toFIXED(6.0))));
  cipiu_is_at_home = !(net_get_team_from_x(g->net,g->n_net,p->pos[X])-(p->team));
  cipiu_has_ball_in_sight = ((adx<toFIXED(70.0))&&(ady<toFIXED(100.0)));

  lim = BALL_SPR_RADIUS+PLAYER_SPR_RADIUS-PLAYER_SPR_CENTER_OFFSET;
  cipiu_is_just_under_the_ball = is_between(lim,dy,lim+toFIXED(30.0));

  lim = slMulFX(toFIXED(-1.0),(BALL_SPR_RADIUS+PLAYER_SPR_RADIUS));
  cipiu_is_just_over_the_ball = is_between(lim-toFIXED(15.0),dy,lim);

  cipiu_is_just_at_right_of_ball = is_between(toFIXED(-20.0),dx,toFIXED(-2.0));
  cipiu_is_just_at_left_of_ball =  is_between(toFIXED(2.0),dx,toFIXED(20.0));


  // ball_falls_without_dx = (b->speed[X]<toFIXED(0.3));

  if (cipiu_guesses_he_wins){

    if (cipiu_has_ball_in_sight){
      if (cipiu_is_just_over_the_ball){
	//	slPrint("xsmatch      ",slLocate(30,3));
	c->target = lop_bal_under;  // smatch
      }else{
	//	slPrint("gosm      ",slLocate(30,3));
	c->target = lop_bal_over;  // go in smatch positions
      }
    } else {
      //      slPrint("stdby    ",slLocate(30,3));
      c->target = lop_a1;        // attack pos 1
    }

  } else {

    if (!cipiu_is_at_home){
      p->options.go_trough_net = (1==0); // get back
    }

    if (!cipiu_is_just_under_the_ball) {
      if (cipiu_guesses_ball_is_too_fast){
	c->target = lop_bal_under_ground;  //defence
	//	slPrint("sous__      ",slLocate(30,3));
      } else {
	c->target = lop_bal_under;  //defence
	//	slPrint("sous    ",slLocate(30,3));
      }
    } else {
      if (cipiu_is_just_at_left_of_ball){
	// paf de gauche a droite
	c->target = lop_bal_top_right;
	//	slPrint("->tr      ",slLocate(30,3));
      } else if (cipiu_is_just_at_right_of_ball){
	// paf de droite a gauche
	c->target = lop_bal_top_left; 
	//	slPrint("->tl   ",slLocate(30,3));
      } else {
	c->target = lop_bal_under_ground;  //defence
	//	slPrint("sous    ",slLocate(30,3));
      }
    }
  }

  // bonuses quick hack.
  /*  if ((c->bonus_timeout)<0) {
    c->fire_bonus = 1;
    } else
  */
  {
  
    int use_prob = 0; // [1..100]
    int rnd_prob = minmaxrnd(1,1000);

    switch (player_options_get_type(&(p->options))){
    case no_bonus:	  
      use_prob = 0;
      break;
    case  proat : 
      use_prob = 25;
      break;
    case throther :   
    case  nosferatu :
      if (cipiu_guesses_he_wins)
	use_prob = 50;
      else 
	use_prob = 5;
      break;
      
      if (cipiu_guesses_he_wins)
	use_prob = 30;
      else 
	use_prob = 6;
      break;
      
    case  kingkong : 
      if ((cipiu_guesses_he_wins)&&(cipiu_guesses_ball_is_too_fast))
	use_prob = 60;
      else 
	use_prob = 0;
      break;
    case kansas: 
      use_prob = 10;
      break;
    case liste_des_fuckings:   
      use_prob = 10;
      break;
    case chomplet:  
      use_prob = 20;
      break;
    case long_drink:   
      use_prob = 6;
      break;
    case jumbo_jim:   
      use_prob = 20;
      break;
    default: 	  
      break;
    }   
    if (use_prob>rnd_prob)
      c->fire_bonus = 1;
  }

  /*
  slPrint("[",slLocate(7,3));
  if (cipiu_guesses_he_wins)   {slPrint("W",slLocate(8,3));} else  {slPrint(" ",slLocate(8,3));}
  if (cipiu_is_at_home)        {slPrint("H",slLocate(9,3));} else  {slPrint(" ",slLocate(9,3));}
  if (cipiu_has_ball_in_sight) {slPrint(":",slLocate(10,3));} else  {slPrint(" ",slLocate(10,3));}
  if (p->options.go_trough_net){slPrint(" ",slLocate(11,3));} else  {slPrint("|",slLocate(11,3));}
  //  if (ball_falls_without_dx)   {slPrint("v",slLocate(12,3));} else {slPrint(" ",slLocate(12,3));}
  switch (cipiu_guesses_where_to_send){
  case 0 : slPrint("0",slLocate(12,3)); break;
  case 1 : slPrint("1",slLocate(12,3));    break;
  case 2:  slPrint("2",slLocate(12,3));    break;
  default : slPrint("?",slLocate(12,3));    break;
  }
  if (cipiu_is_just_over_the_ball) {slPrint("O",slLocate(13,3));} else  {slPrint("|",slLocate(13,3));}
  if (cipiu_is_just_under_the_ball) {slPrint("U",slLocate(14,3));} else  {slPrint("|",slLocate(14,3));}
  if (cipiu_is_just_at_right_of_ball) {slPrint("L",slLocate(15,3));} else  {slPrint("|",slLocate(15,3));}
  if (cipiu_is_just_at_left_of_ball) {slPrint("R",slLocate(16,3));} else  {slPrint("|",slLocate(16,3));}
  if (cipiu_guesses_ball_is_too_fast) {slPrint("f",slLocate(17,3));} else  {slPrint(" ",slLocate(17,3));}
  slPrint("]",slLocate(18,3));
  */
}

/* 
 * transforms logical target position into a movement
 */
void cipiu_move(CIPIU* c, GAME* g, BALL* bs){

  FIXED t_pos[XY];  // target p
  FIXED ft_pos[XY]; // target p future position
  FIXED r_pos[XY];  // relative position
  LOGICAL_POS l_pos = c->target;
  BALL* b = (BALL*) bs;
  
  ft_pos[X] = b->pos[X]+slMulFX(toFIXED(10.0),b->speed[X]);
  ft_pos[Y] = (b->speed[X]>toFIXED(0.0))?(b->pos[Y]+slMulFX(toFIXED(5.0),b->speed[Y])):(b->pos[Y]);

  // get target (x,y) pos
  switch (l_pos){
  case lop_bal_over : 
    t_pos[X] = ft_pos[X];
    if ((b->speed[Y])<toFIXED(1.0)){
      t_pos[Y] = ft_pos[Y]-BALL_SPR_RADIUS-PLAYER_SPR_RADIUS;
    } else {
      t_pos[Y] = ft_pos[Y];
    }
    break;
  case lop_bal_under : 
    t_pos[X] = ft_pos[X];//b->pos[X]+minmaxrnd(toFIXED(-3.0),toFIXED(3.0));
    t_pos[Y] = BALL_SPR_RADIUS-PLAYER_SPR_RADIUS+toFIXED(30.0); 
    break;
  case lop_bal_under_ground : 
    t_pos[X] = ft_pos[X];
    t_pos[Y] = POS_BOTTOM;
    break;
  case lop_bal_top_left :
    t_pos[X] = b->pos[X] - toFIXED(3.0);
    t_pos[Y] = b->pos[Y];
    break;
  case lop_bal_top_right :
    t_pos[X] = b->pos[X] + toFIXED(3.0);
    t_pos[Y] = b->pos[Y];
    break;
  case lop_a1 : 
  default :
    t_pos[X] = c->default_pos[X];
    t_pos[Y] = c->default_pos[Y];
    break;
  }

  // display target
  /*  {
    static ANGLE ang = 0;
    FIXED pos[XYZS];
    int spr_num = TARGET_SPR;
    
    pos[X] = t_pos[X];
    pos[Y] = t_pos[Y];
    pos[Z] = toFIXED(169.0);
    pos[S] = toFIXED(0.25);
    
    ang +=DEGtoANG(30.0);
    slDispSprite(pos, (SPR_ATTR *)(&attr[spr_num].texno), ang);   
  }
  */
  r_pos[X] = c->p->pos[X] - t_pos[X];
  r_pos[Y] = c->p->pos[Y] - t_pos[Y];

  // move to this target
  if (r_pos[X]>toFIXED(0.0)){
    cipiu_muscle_control(c,cc_left);
  } else {
    cipiu_muscle_control(c,cc_right);
  }  

  if ((r_pos[Y]>toFIXED(0.0))&&(c->p->speed[Y])>toFIXED(-0.1)){
    cipiu_muscle_control(c,cc_up);
  }

  if (c->fire_bonus){
    cipiu_muscle_control(c,cc_b);
  }

}

/*
 * Effective controls
 */


void cipiu_muscle_control(CIPIU* c, CIPIU_MUSCLE_CONTROL cc){

  PLAYER* p = c->p;
  
  //  slPrintFX(toFIXED(1.0*(c->muscle_timeout[cc])),slLocate(10,10+cc));
 
  if ((c->muscle_timeout[cc])==0){
    switch (cc){
    case cc_left  : 
      p->speed[X] -= toFIXED(1.0);
      c->muscle_timeout[cc] = 0;
      break;
    case cc_right : 
      p->speed[X] += toFIXED(1.0); 
      c->muscle_timeout[cc] = 0;
      break;
    case cc_up    : 
      p->speed[Y] = toFIXED(-3.0); 
      c->muscle_timeout[cc] = 30;
      player_give_gaz(p);
      break;
    case cc_b :
      c->muscle_timeout[cc] = 50;
      c->bonus_timeout = minmaxrndint(90,CIPIU_BONUS_TIMEOUT) ;
    default : 
      break;
    }
  }
}










