/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
 *  simplistic quasi-tree-like finite state automata 
 *  for pad combo detection
 * 
 * limitations :
 * - transition from state A to B is done if the transition function's event 
 *  (ie, expected pad data) and event (ie, actual pad data) are exactly the
 *  same
 */
#include "combofsa.h"
#include "scrltxt.h"

/*
  an alphabet 
  a set of states
  a transition function
  a start state
  a set of accept states
*/



/* //--------------------------------- */
/* while (;;){ */
/*   automata_feed(a, Smpc_Peripheral[0].pdata); */
/*   automata_step(a); */
/*   switch(automata_get_state_id()){ */
/*   case jmp: // do something */
/*     break; */
/*   case hkick: */
/*     break; */
/*   default: */
/*   } */
 
/* } */

//---------------------------------

// timer
inline void timer_arm(TIMER* t, FIXED time) {t->time = time;}
inline void timer_step(TIMER* t) {t->time = t->time - 1 ; }
inline int  timer_now(TIMER* t) {return (t->time <= 0);}

// transition
void transition_define(TRANSITION* tr, Uint32 pdata, STATE* st){
  tr->pdata = pdata;
  tr->st = st;
}

// state
void state_define(STATE* st, int id, int max_time){
  st->id = id;
  st->trans = (TRANSITION**) NULL;
  st->n_trans = 0;
  st->max_time = max_time;
}

void state_set_transitions(STATE* st, TRANSITION** tr, int n_tr){
  st->trans = tr;
  st->n_trans = n_tr;
}

// automata
void automata_define(AUTOMATA* au, STATE* start){
  au->start = start;
  automata_reset(au);
}

void automata_reset(AUTOMATA* au){
  automata_goto_step(au,au->start);
  au->pdata = 0;
}

void automata_feed(AUTOMATA* au, Uint32 paddata){
  au->pdata = paddata;
}

void automata_step(AUTOMATA* au ){
  TIMER* t = &(au->timer);
  int neutral = ((au->cur_state)==(au->start));

  if (timer_now(t)) {
    if (!neutral){
      automata_goto_step(au,au->start);
    }
  } else {
    STATE* st = au->cur_state;
    int i = 0;
    while (i<st->n_trans){
      TRANSITION* tr = st->trans[i];
      Uint16 padd = ~(au->pdata);
      if (padd & (tr->pdata)){
	automata_goto_step(au,tr->st);
	break;
      }
      i++;
    }
    if (!neutral) timer_step(t);
  }
}

int automata_get_state_id(AUTOMATA* au){
  return (au->cur_state->id);
}

void automata_goto_step(AUTOMATA* au, STATE* st){
  au->cur_state = st;
  //  scrollTxtAddTxtLine("au::gotostep");
  timer_arm(&(au->timer),st->max_time);
}




 
