/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
 *  simplistic quasi-tree-like finite state automata 
 *  for pad combo detection
 */
#ifndef _COMBOFSA_H
#define _COMBOFSA_H

#include "sgl.h"

// simple timer object
typedef struct _TIMER {
  int time;
} TIMER;
void timer_arm(TIMER* t, FIXED time);
void timer_step(TIMER* t);
int  timer_now(TIMER* t);

/* cross-ref between structures */
/* struct _B; */
/* typedef struct { */
/*   struct _B* brou; */
/* } A; */
/* struct _B { */
/*   A* brou; */
/* } ; */
/* typedef struct _B B; */

// automata strctures

typedef struct {
  Uint32 pdata;          // padata to be matched to go to ..
  struct _STATE *st;     // .. this state
} TRANSITION;

struct _STATE {
  int id;                // optional id of the state (surely part of a typedef enum)
  TRANSITION** trans;    // associated transition table
  int n_trans;           // ^ its size
  int max_time;          // nb of step before the automata resets (back to neutral)
} ;

typedef struct _STATE STATE;

typedef struct _AUTOMATA {  
  STATE* start;          // starting state ("neutral")
  STATE* cur_state;      // current state
  TIMER timer;           // current timer count
  Uint32 pdata;          // current pad data
} AUTOMATA;

// automata definition
void transition_define(TRANSITION* tr, Uint32 pdata, STATE* st);
void state_define(STATE* st, int id, int max_time);
void state_set_transitions(STATE* st, TRANSITION** tr, int n);
void automata_define(AUTOMATA* au, STATE* start_state);

// automata usage
void automata_feed(AUTOMATA* au, Uint32 paddata);
void automata_step(AUTOMATA* au );
int  automata_get_state_id(AUTOMATA* au);
void automata_reset(AUTOMATA* au);

// "internal" : shouldn't be exported
void automata_goto_step(AUTOMATA* au, STATE* st);

#endif
