/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef GAME_OBJ_H
#define GAME_OBJ_H

#include "sgl.h"
#include "combofsa.h"
#include "cipiu.h"

#define MAX_BONUS_NB 10

/*
 ************** GAME OBJECTS
 */

// EXCHANGE
typedef struct _EXCHANGE_RESULT {
  int losing_team;     // winning team for this exchange
  int next_distrib;
} EXCHANGE_RESULT;

void init_exchange_resu(EXCHANGE_RESULT* e, FIXED next_distrib);

// NET
typedef struct _NET {
  FIXED pos[XYZS];      // where the net is
} NET;

void net_init(NET* n, FIXED posX);
void net_display(NET* n);
int net_get_team_from_x(NET* net, int n_net, FIXED x);

// TEAM
typedef struct _TEAM {
  int points;   
} TEAM;

void team_init(TEAM* t);


// GAME_OPTIONS & PLAYER_OPTIONS (g and pl specific)

/*
  blue - kingkong - the ball bounces on the ground
  green - nosferatu - the ball does not collide with players
  big purple - long_drink - creates a fake ball
  big gray - chomplet - freezes the ball
  gray - liste des fuckings - makes the ball invisible
  yellow - throter - make the net full heighted
  orange - kansas - makes the player go down
  purpler - proat - makes the ball go up
  bug red - jumbo_jim - disables boucing on the top of the screen
 */


typedef enum _E_BONUS {super_jump=0,floupix,kingkong,throther,kansas,
			 little_kansas,nosferatu,proat,liste_des_fuckings,
			 side_kick,jumbo_jim,chomplet,long_drink,no_bonus} E_BONUS; 

typedef struct _PLAYER_OPTIONS {
  int go_trough_net; 
  E_BONUS queue[MAX_BONUS_NB];
  FIXED pos[MAX_BONUS_NB][XYZS];
} PLAYER_OPTIONS;


void display_bonus_sprite(FIXED* pos, ANGLE ang, E_BONUS type);
void player_options_init(PLAYER_OPTIONS* op);
int player_options_queue(PLAYER_OPTIONS* op, E_BONUS type, int max);
E_BONUS player_options_get_type(PLAYER_OPTIONS* op);
E_BONUS player_options_pop(PLAYER_OPTIONS* op);
void player_options_move(PLAYER_OPTIONS* op, FIXED *target);
void player_options_display(PLAYER_OPTIONS* op);

// DISTRIBUTION ! ok_corral  must be the last (for options.c)
typedef enum _DISTRIBUTION_RULE {none,straight,smooth,boumtou,chainsaw,ok_corral} DISTRIBUTION_RULE;
// CONTROL
typedef enum _CONTROL_SET {cs_standard,cs_floupix_con,cs_combo,cs_slime} CONTROL_SET;
// GAME LENGHT
typedef enum _GAME_LENGHT {gl_flash,gl_short,gl_medium,gl_long,gl_very_long,gl_endless,gl_sysiphe} GAME_LENGHT;
// BONUS_COLLECTION
/* 
   normal : all but green and light blue
   xaporu : purple and yellow
   blake  : whitout little grey
   bali-bali : purple green big grey
   lubion : only orange
   glarpi : purple and red
   power  : all  
*/

typedef enum _BONUS_COLLECTION {bc_normal=0, bc_xaporu, bc_blake, bc_balibali, bc_lubion, bc_glarpi, bc_power} BONUS_COLLECTION;

#define BONUS_PROBA_SIZE 15
E_BONUS bonus_proba_get(BONUS_COLLECTION bc, int cell);


// MUSIC
typedef enum _CD_MUSIC {cdmusic_on, cdmusic_off} CD_MUSIC;

// GAME_OPTIONS
typedef struct _GAME_OPTIONS {
  int instant_match;
  DISTRIBUTION_RULE distrib;
  CONTROL_SET control_set;
  GAME_LENGHT game_lenght;
  BONUS_COLLECTION bonus_collection;
  CD_MUSIC cd_music;
  int is_on[no_bonus]; // no_bonus is the nb of the last bonus...
} GAME_OPTIONS;

void game_options_init(GAME_OPTIONS* gop);
int  game_options_get_max_point(GAME_OPTIONS* gop);

#define IS_CIPIU 0xff

//  PLAYER
typedef struct _PLAYER {
  int pad_num;              // set to IS_CIPIU if ia controls the player.
  FIXED pos[XYZS];
  FIXED last_pos[XY];
  FIXED speed[XYZ];
  FIXED mass;
  int team;
  int sprite;
  int gaz_counter;
  AUTOMATA pad_fsa;
  PLAYER_OPTIONS options;
} PLAYER;

void player_init(PLAYER* p, int pad_num, FIXED* pos,  FIXED mass, int team, int sprite);
void player_reinit(PLAYER* p);

void player_display(PLAYER* p);
void player_give_gaz(PLAYER* p);
// BALL
typedef struct _BALL {
  FIXED pos[XYZS];
  FIXED last_pos[XY];
  FIXED speed[XYZ];
  FIXED mass;
} BALL;

void ball_init(BALL* b,FIXED* pos, FIXED* speed, FIXED mass);
void ball_display(BALL* b);
void ball_dbg_display_speed(BALL* b,int x, int y);

// GAME
typedef struct _GAME {
  TEAM *team;
  int n_team;
  PLAYER *player;
  int n_player;
  CIPIU *cipiu;
  int n_cipiu;
  NET *net;
  int n_net;
  PLAYER_OPTIONS  pl_options;
  GAME_OPTIONS*   ga_options;
} GAME;

void game_init(GAME* g, TEAM *team, int n_team, PLAYER *player, int n_player, CIPIU *cipiu, int n_cipiu, NET *net, int n_net, GAME_OPTIONS* ga_options);

#endif


