/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "persist.h"
#include "sgl.h"
#include "sega_bup.h" // Required for backup memory functions

extern int sprintf (char *, const char *, ...);
extern char * strcpy(char *, const char *);

#define BUP_START_ADDR 0x6070000 // This is where the backup library will expand to
#define FLOUPIX_SAVE_NAME "floupix"

Uint32	BackUpRamWork[2048]; // Needed for BackUpFunctions
Uint8   BackUpLibSpace[16384]; // Needed for BackUpFunctions

#define FL_SAV_VERSION 1

Sint32 backup_file_read(Uint32 device, char* filename, Uint8* image, int image_size);
Sint32 backup_file_write(Uint32 device, char* filename, Uint8* image, int image_size);

/*
 * Floupix related
 */
#define SAV_LENGHT 6

// saves things into backup memory

int persist_set(GAME_OPTIONS* gop){

  Uint8 image[SAV_LENGHT];
  Sint32 resu;

  image[0] = (Uint8) FL_SAV_VERSION;
  image[1] = (Uint8) gop->distrib;
  image[2] = (Uint8) gop->control_set;
  image[3] = (Uint8) gop->game_lenght;
  image[4] = (Uint8) gop->bonus_collection;
  image[5] = (Uint8) gop->cd_music;
  
  resu = backup_file_write(0, FLOUPIX_SAVE_NAME, image, SAV_LENGHT);
  return resu;
}

// read the saves from backup and pour into game (options)
int persist_get(GAME_OPTIONS* gop){
  Uint8 image[SAV_LENGHT];
  int  resu=-1;
  resu = backup_file_read(0, FLOUPIX_SAVE_NAME,image, SAV_LENGHT);
  if (!resu){
    if (image[0] == 1){  // version 1
      gop->distrib = (int) image[1];
      gop->control_set = (int) image[2];
      gop->game_lenght = (int) image[3];
      gop->bonus_collection = (int) image[4];
      gop->cd_music = (int) image[5];
    } else {
      resu = -1;
    }
  }
  return (resu);
}

/*
 * Generic
 */

void	BackUpInit(BupConfig cntb[3] );
Sint32	BackUpWrite(Uint32 device, BupDir *dir, Uint8 *data, Uint8 sw);
Sint32	BackUpDelete(Uint32 device,Uint8 *filename);
Sint32	BackUpFormat(Uint32 device);

Sint32 backup_file_read(Uint32 device, char* filename, Uint8* image, int image_size){

  BupConfig conf[3]; // stores backup configuration
  BupStat sttb;      // stores backup status
  Sint32 status;

  // intialize backup
  BackUpInit(conf);

  // device is present, format, etc ?
  status = BUP_Stat(device, image_size, &sttb);
  if (status){
    BUP_Read(device,(Uint8*)filename,image);
  }
  return status;

}

Sint32 backup_file_write(Uint32 device, char* filename, Uint8* image, int image_size){
  BupConfig conf[3]; // stores backup configuration
  BupStat sttb;      // stores backup status
  BupDir writetb;    // writeb holds file data
  Sint32 status;

  // intialize backup
  BackUpInit(conf);

  // device is present ?
  status = BUP_Stat(device, image_size, &sttb);

  // device is formated ?
  if (status == BUP_UNFORMAT){
    BackUpFormat(device); // if not, format...
    status = BUP_Stat(device, image_size, &sttb ); // .. and update status
  }

  // eventualy ok for write access ?
  if (status){     
   // Puts the proper information in writetb
    strcpy((char *)writetb.filename, filename);
    strcpy((char *)writetb.comment, "");
    writetb.language = BUP_FRANCAIS;
    writetb.datasize = image_size;
    status = BUP_Write(device, &writetb, image, OFF);
  }    
  return status;
}

/*
 * functions taken from sgl example "sample2.c"
 */

void	BackUpInit(BupConfig cntb[3] )
{
	slResetDisable();
	//	BUP_Init((Uint32 *)BUP_START_ADDR,BackUpRamWork,cntb);
	BUP_Init((Uint32 *)BackUpLibSpace,BackUpRamWork,cntb);
	slResetEnable();
}

Sint32	BackUpWrite(Uint32 device, BupDir *dir, Uint8 *data, Uint8 sw)
{
	Sint32		ret;
	SmpcDateTime	*time;
	BupDate		date;
	
	if(!dir->date){
		time = &(Smpc_Status->rtc);
		date.year  = (Uint8 )( slDec2Hex((Uint32)time->year)-1980 ); /* Modify K.T */
		date.month = (Uint8 )( time->month & 0x0f);
		date.week  = (Uint8 )( time->month >> 4  );
		date.day   = (Uint8 )( slDec2Hex((Uint32)time->date));   /* Modify K.T */
		date.time  = (Uint8 )( slDec2Hex((Uint32)time->hour));   /* Modify K.T */
		date.min   = (Uint8 )( slDec2Hex((Uint32)time->minute)); /* Modify K.T */
		dir->date = BUP_SetDate(&date);
	}
	slResetDisable();
	ret = BUP_Write(device,dir,data,sw);
	slResetEnable();

	return(ret);
}

Sint32	BackUpDelete(Uint32 device,Uint8 *filename)
{
	Sint32	ret;

	slResetDisable();
	ret = BUP_Delete(device,filename);
	slResetEnable();

	return(ret);
}


Sint32	BackUpFormat(Uint32 device)
{
	Sint32	ret;

	slResetDisable();
	ret = BUP_Format(device);
	slResetEnable();

	return(ret);
}


