/*

floupix ping pong - A multiplayer game for saturn
Copyright (C) 2004 SSMTF

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PATICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "physics.h"
#include "scrltxt.h"
/*
 ****************** physics
 */

void world_init(WORLD* w, FIXED g, FIXED gf, FIXED af, FIXED sl){
  w->gravity = g;   
  w->ground_friction = gf;  
  w->air_friction = af;
  w->speed_limit = sl;
}

void slow_speed(FIXED* speed, FIXED friction){
  FIXED sx, sy, norm, fric_norm, cos_a, sin_a;

  sx = speed[X];
  sy = speed[Y];

  norm = slSquartFX (slMulFX(sx,sx)+slMulFX(sy,sy));

  cos_a = slDivFX(norm,sx);
  sin_a = slDivFX(norm,sy);

  fric_norm = slMulFX(norm, friction);
  
  speed[X] = slMulFX(fric_norm, cos_a);
  speed[Y] = slMulFX(fric_norm, sin_a);
}

void limit_speed(FIXED* speed, FIXED limit){
  FIXED sx, sy, norm, cos_a, sin_a;
  
  sx = speed[X];
  sy = speed[Y];

  norm = slSquartFX(slMulFX(sx,sx)+slMulFX(sy,sy));

  cos_a = slDivFX(norm,sx);
  sin_a = slDivFX(norm,sy);

  if (norm>limit) { 
    speed[X] = slMulFX(limit, cos_a);
    speed[Y] = slMulFX(limit, sin_a);
  }
}


// alter speed according to the gravity
// (only if object is NOT on the ground, )
void apply_gravity(FIXED* pos, FIXED* speed, FIXED mass, WORLD* world){
    speed[Y] += slMulFX(world->gravity,mass);
}

// reduce speed according to friction
// (called only if object is on the ground)
void apply_ground_friction(FIXED* pos, FIXED* speed, WORLD* world){
    speed[X] = slMulFX(speed[X], world->ground_friction);
}


// reduce speed according to friction
// (called only if object is in the air)
void apply_air_friction(FIXED* pos, FIXED* speed, WORLD* world){
  slow_speed(speed, world->air_friction);
}


// change the speed according to world->speed_limit
void apply_speed_limit(FIXED* pos, FIXED* speed, WORLD* world){
  limit_speed(speed, world->speed_limit);
}


// alter position according to speed
void apply_speed(FIXED* pos, FIXED* speed, WORLD* world){
  int i;
  for (i=0;i<XYZ;i++){
    pos[i] += speed[i];
  }
}

